module Main (main, part1, part2) where
import qualified Text.Parsec as P
import Text.Parsec.String (Parser)
import Text.Printf (printf)
import Control.Monad (guard)
import Data.Either (rights)
import Control.Monad (sequence)

parseInt :: String -> Int
parseInt = read

passwordParser :: (Int -> Int -> Char -> String -> Bool) -> Parser String
passwordParser g = do
  lower <- parseInt <$> P.many1 P.digit
  P.char '-'
  upper <- parseInt <$> P.many1 P.digit
  P.char ' '
  c <- P.letter
  P.string ": "
  password <- P.many1 P.letter
  guard (g lower upper c password)
  return password

part1Parser =
  passwordParser guardF
  where guardF lower upper c password =
          let count = characterCount c password
          in lower <= count && count <= upper
        characterCount c = length . filter (c==)

part2Parser =
  passwordParser guardF
  where guardF lower upper c password =
          let lowerC = password !! (lower - 1)
              upperC = password !! (upper - 1)
          in (c == lowerC || c == upperC) && lowerC /= upperC

solve :: Parser String -> [String] -> Int
solve parser = length . rights . map (P.parse parser "")

part1 = solve part1Parser
part2 = solve part2Parser

main :: IO ()
main = do
  contents <- lines <$> readFile "input"
  putStr $ printf "Part 1: %d\n" (part1 contents)
  putStr $ printf "Part 2: %d\n" (part2 contents)
