(load (merge-pathnames "../common.lisp" *load-truename*))

(defun to-array (data)
  (make-array (list (length data)
		    (length (car data)))
	      :initial-contents data))

(defconstant test-data
	     '(
	       "..##......."
	       "#...#...#.."
	       ".#....#..#."
	       "..#.#...#.#"
	       ".#...##..#."
	       "..#.##....."
	       ".#.#.#....#"
	       ".#........#"
	       "#.##...#..."
	       "#...##....#"
	       ".#..#...#.#"
	       ))

(defconstant tree #\#)

(defun path (map next-x next-y)
  (let ((max-y (car (array-dimensions map)))
        (max-x (cadr (array-dimensions map))))
    (loop for x = 0 then (mod (+ next-x x) max-x)
          for y = 0 then (+ next-y y)
          while (< y max-y)
	  collect (aref map y x))))

(defun trees-on-path (path)
  (loop for step in path counting (char= tree step)))

(defun solution (input variants)
  (apply #'*
         (loop for variant in variants
               collect (trees-on-path (path input (car variant) (cdr variant))))))

(defun part1 (input)
  (solution input '((3 . 1))))

(defun part2 (input)
  (solution input '((1 . 1) (3 . 1) (5 . 1) (7 . 1) (1 . 2))))

(let ((input (to-array test-data)))
  (assert (= 7 (part1 input)))
  (assert (= 336 (part2 input))))

(let ((input (to-array (read-data))))
  (format t "Part 1: ~A~%" (part1 input))
  (format t "Part 2: ~A~%" (part2 input)))
