(load (merge-pathnames "../common.lisp" *load-truename*))

(defun binary-search (str down-char)
  (labels ((next (start end) (- end (/ (- end start) 2))))
    (loop for c across str
          with i = 0
          with j = (expt 2 (length str))
          do (let ((next-i (next i j)))
               (if (char= down-char c)
                   (setf j next-i)
                   (setf i next-i)))
          finally (return i))))

(defun row (str)
  (binary-search (subseq str 0 7) #\F))

(defun column (str)
  (binary-search (subseq str 7) #\L))

(defun id (str)
  (+ (column str)
     (* (row str) 8)))

(let ((test-data "FBFBBFFRLR"))
  (assert (= 44 (row test-data)))
  (assert (= 5 (column test-data)))
  (assert (= 357 (id test-data))))

(let ((test-data "BFFFBBFRRR"))
  (assert (= 70 (row test-data)))
  (assert (= 7 (column test-data)))
  (assert (= 567 (id test-data))))

(let ((test-data "FFFBBBFRRR"))
  (assert (= 14 (row test-data)))
  (assert (= 7 (column test-data)))
  (assert (= 119 (id test-data))))

(let ((test-data "BBFFBBFRLL"))
  (assert (= 102 (row test-data)))
  (assert (= 4 (column test-data)))
  (assert (= 820 (id test-data))))

(defun part1 (input)
  (loop for line in input
        maximize (id line)))

(defun part2 (input)
  (let* ((ids (mapcar 'id input))
         (sorted (sort ids #'<)))
    (loop for pair in (mapcar 'list sorted (cdr sorted))
          when (= 2 (abs (apply '- pair)))
            return (1+ (apply 'min pair)))))

(let ((input (read-data)))
  (format t "Part 1: ~A~%" (part1 input))
  (format t "Part 2: ~A~%" (part2 input)))
