(load (merge-pathnames "../common.lisp" *load-truename*))

(defconstant test-data
  '("light red bags contain 1 bright white bag, 2 muted yellow bags."
    "dark orange bags contain 3 bright white bags, 4 muted yellow bags."
    "bright white bags contain 1 shiny gold bag."
    "muted yellow bags contain 2 shiny gold bags, 9 faded blue bags."
    "shiny gold bags contain 1 dark olive bag, 2 vibrant plum bags."
    "dark olive bags contain 3 faded blue bags, 4 dotted black bags."
    "vibrant plum bags contain 5 faded blue bags, 6 dotted black bags."
    "faded blue bags contain no other bags."
    "dotted black bags contain no other bags."))

(defconstant test-data-2
  '("shiny gold bags contain 2 dark red bags."
    "dark red bags contain 2 dark orange bags."
    "dark orange bags contain 2 dark yellow bags."
    "dark yellow bags contain 2 dark green bags."
    "dark green bags contain 2 dark blue bags."
    "dark blue bags contain 2 dark violet bags."
    "dark violet bags contain no other bags."))

(defun words (string)
  (loop for i = 0 then (1+ j)
        as j = (position #\Space string :start i)
        collect (subseq string i j)
        while j))

(defun parse-allowed-contents (contents)
  (if (string= "no" (car contents))
      '()
      (let ((length (length contents)))
        (loop for i = 0 then (+ i 4)
              while (< (1+ i) length)
              collect (cons (color-from-parts (subseq contents (1+ i)))
                            (parse-integer (nth i contents)))))))

(defun color-from-parts (parts)
  (concatenate 'string (car parts) " " (cadr parts)))

(defun parse-line (line)
  (let ((parts (words line)))
    (values (color-from-parts parts) (parse-allowed-contents (subseq parts 4)))))

(defun parse-input (input)
  (labels ((reduction (acc cur)
             (multiple-value-bind (key values)
                 (parse-line cur)
               (acons key values acc))))
    (reduce #'reduction input
            :initial-value '())))

(defun can-contain (rules wanted)
  (remove-if-not (lambda (rule)
                   (assoc wanted (cdr rule) :test #'string=))
                 rules))

(defun transitive-can-contain (rules wanted &optional (acc '()))
  (let* ((found (mapcar #'car (can-contain rules wanted)))
         (new (remove-if (lambda (x) (member x acc :test #'string=)) found))
         (acc (concatenate 'list new acc)))
    (reduce
     (lambda (acc cur) (transitive-can-contain rules cur acc))
     new
     :initial-value acc)))

(defun bag-count (rules wanted)
  (reduce (lambda (acc cur)
            (+ acc (* (cdr cur) (bag-count rules (car cur)))))
          (cdr (assoc wanted rules :test #'string=))
          :initial-value 1))

(defun part1 (input)
  (length (transitive-can-contain (parse-input input) "shiny gold")))

(defun part2 (input)
  (1- (bag-count (parse-input input) "shiny gold")))

(assert (= 4 (part1 test-data)))
(assert (= 126 (part2 test-data-2)))

(let ((input (read-data)))
  (format t "Part 1: ~A~%" (part1 input))
  (format t "Part 2: ~A~%" (part2 input)))
