(load (merge-pathnames "../common.lisp" *load-truename*))

(defconstant test-data
  '("nop +0"
    "acc +1"
    "jmp +4"
    "acc +3"
    "jmp -3"
    "acc -99"
    "acc +1"
    "jmp -4"
    "acc +6"))

(defun execute (program)
  (let ((program (make-array (length program) :initial-contents program))
        (hits (make-array (length program))))
    (loop with ip = 0
          with acc = 0
          while (and (< ip (length program)) (< (aref hits ip) 1))
          do (let ((instruction (aref program ip)))
               (incf (aref hits ip))
               (cond
                 ((string= "nop" (car instruction)) (setf ip (1+ ip)))
                 ((string= "acc" (car instruction)) (setf ip (1+ ip)
                                                          acc (+ acc (cdr instruction))))
                 ((string= "jmp" (car instruction)) (setf ip (+ ip (cdr instruction))))))
          finally (return (values acc (if (= ip (length program)) 'clean 'loop))))))

(defun parse-input (input)
  (mapcar
   (lambda (e) (cons (subseq e 0 3) (parse-integer (subseq e 4))))
   input))

(assert (= 5 (execute (parse-input test-data))))

(defun part1 (input)
  (execute (parse-input input)))

(defun with-replacement (source position new-value)
  (loop for e in source
        for i from 0 to (length source)
        collect (if (= i position) (cons new-value (cdr e)) e)))

(defun part2 (input)
  (let ((input (parse-input input)))
    (loop named top
          for replacement in '(("nop" . "jmp") ("jmp" . "nop"))
          do (loop for i = 0 then (1+ j)
                   as j = (position (car replacement) input :start i :key #'car :test #'string=)
                   while j
                   do (multiple-value-bind (value result)
                          (execute (with-replacement input j (cdr replacement)))
                        (when (eq 'clean result)
                          (return-from top value)))))))

(assert (= 8 (part2 test-data)))

(let ((input (read-data)))
  (format t "Part 1: ~A~%" (part1 input))
  (format t "Part 2: ~A~%" (part2 input)))
