(load (merge-pathnames "../common.lisp" *load-truename*))

(defun parse-input (input)
  (mapcar #'parse-integer input))

(defconstant test-data
  (parse-input '(
                 "16"
                 "10"
                 "15"
                 "5"
                 "1"
                 "11"
                 "7"
                 "19"
                 "6"
                 "12"
                 "4"
                 )))

(defconstant test-data-1
  (parse-input
   '("28"
     "33"
     "18"
     "42"
     "31"
     "14"
     "46"
     "20"
     "48"
     "47"
     "24"
     "23"
     "49"
     "45"
     "19"
     "38"
     "39"
     "11"
     "1"
     "32"
     "25"
     "35"
     "8"
     "17"
     "7"
     "9"
     "4"
     "2"
     "34"
     "10"
     "3")))


(defun device-joltage (adapters)
  (+ 3 (apply #'max adapters)))

(assert (= 22 (device-joltage test-data)))

(defun adapter-differences (input)
  (let* ((device (device-joltage input))
         (joltages (cons 0 (cons device input)))
         (chain (sort (copy-list joltages) #'<)))
    (loop for x in chain
          for y in (cdr chain)
          counting (= 1 (- y x)) into ones
          counting (= 3 (- y x)) into threes
          finally (return (list ones threes)))))

(assert (equal (list 7 5)
               (adapter-differences test-data)))
(assert (equal (list 22 10)
               (adapter-differences test-data-1)))

(defun part1 (input)
  (apply #'* (adapter-differences input)))

(defun with-memo (memo f key)
  (let ((memoized (aref memo key)))
    (if memoized
        memoized
        (let ((calculated (funcall f key)))
          (setf (aref memo key) calculated)
          calculated))))

(defun chains (start options target memo)
  (labels ((possibilities ()
             (remove-if-not (lambda (e) (and (<= e (+ 3 start))
                                             (>= e (1+ start)))) options))
           (calculate-chains (start)
             (let ((possibilities (possibilities)))
               (cond
                 ((= start target) 1)
                 ((null possibilities) 0)
                 (t (apply #'+ (mapcar
                                (lambda (x) (chains x options target memo))
                                possibilities)))))))
    (with-memo memo #'calculate-chains start)))

(defun part2 (input)
  (let* ((joltage (device-joltage input))
         (memo (make-array (1+ joltage) :initial-element nil)))
    (chains 0 (cons 0 (cons joltage input)) joltage memo)))

(assert (= 8 (part2 test-data)))
(assert (= 19208 (part2 test-data-1)))

(let ((input (parse-input (read-data))))
  (format t "Part 1: ~A~%" (part1 input))
  (format t "Part 2: ~A~%" (part2 input)))
