(load (merge-pathnames "../common.lisp" *load-truename*))

(defstruct coordinate (x 0) (y 0))

(defstruct ship (coordinate (make-coordinate)) (bearing 'east))

(defconstant bearings '(north east south west))

(defconstant test-data '("F10"
                         "N3"
                         "F7"
                         "R90"
                         "F11"))

(defun parse-line (line)
  (cons (aref line 0) (parse-integer (subseq line 1))))

(defun advance (coordinate x y)
  (setf (coordinate-x coordinate) (+ x (coordinate-x coordinate))
        (coordinate-y coordinate) (+ y (coordinate-y coordinate))))

(defun forward (ship amount)
  (case (ship-bearing ship)
    (north (advance (ship-coordinate ship) amount 0))
    (east  (advance (ship-coordinate ship) 0 amount))
    (south (advance (ship-coordinate ship) (- amount) 0))
    (west  (advance (ship-coordinate ship) 0 (- amount)))))

(defun normalize-turn (direction amount)
  (let ((normalized (mod amount 360)))
    (/ (if (eq direction 'right)
           normalized
           (- 360 normalized))
       90)))

(defun next-bearing (bearing amount)
  (nth (mod (+ amount (position bearing bearings)) 4) bearings))

(defun turn (ship amount)
  (setf (ship-bearing ship) (next-bearing (ship-bearing ship) amount)))

(defun move (ship instruction)
  (case (car instruction)
    (#\L (turn ship (normalize-turn 'left (cdr instruction))))
    (#\R (turn ship (normalize-turn 'right (cdr instruction))))
    (#\N (advance (ship-coordinate ship) (cdr instruction) 0))
    (#\S (advance (ship-coordinate ship) (- (cdr instruction)) 0))
    (#\E (advance (ship-coordinate ship) 0 (cdr instruction)))
    (#\W (advance (ship-coordinate ship) 0 (- (cdr instruction))))
    (#\F (forward ship (cdr instruction)))))

(defun rotate-coordinate (coordinate amount)
  (flet ((update-coordinate (x y)
           (setf (coordinate-x coordinate) x
                 (coordinate-y coordinate) y)))
    (let ((x (coordinate-x coordinate))
          (y (coordinate-y coordinate)))
      (case amount
        (0 (update-coordinate x y))
        (1 (update-coordinate (- y) x))
        (2 (update-coordinate (- x) (- y)))
        (3 (update-coordinate y (- x)))))))

(defun manhattan-distance (ship)
  (let ((position (ship-coordinate ship)))
    (+ (abs (coordinate-x position))
       (abs (coordinate-y position)))))

(defun part1 (input)
  (loop with ship = (make-ship)
        for instruction in (mapcar #'parse-line input)
        do (move ship i)
        finally (return (manhattan-distance ship))))

(defun move-coordinate (ship waypoint instruction)
  (case (car instruction)
    (#\L (rotate-coordinate waypoint (normalize-turn 'left (cdr instruction))))
    (#\R (rotate-coordinate waypoint (normalize-turn 'right (cdr instruction))))
    (#\N (advance waypoint (cdr instruction) 0))
    (#\S (advance waypoint (- (cdr instruction)) 0))
    (#\E (advance waypoint 0 (cdr instruction)))
    (#\W (advance waypoint 0 (- (cdr instruction))))
    (#\F (advance (ship-coordinate ship)
                  (* (cdr instruction) (coordinate-x waypoint))
                  (* (cdr instruction) (coordinate-y waypoint))))))

(defun part2 (input)
  (loop with ship = (make-ship)
        with coordinate = (make-coordinate :x 1 :y 10)
        for instruction in (mapcar #'parse-line input)
        do (move-coordinate ship coordinate instruction)
        finally (return (manhattan-distance ship))))

(let ((input (read-data)))
  (format t "Part 1: ~A~%" (part1 input))
  (format t "Part 2: ~A~%" (part2 input)))
