(use-modules (ice-9 textual-ports)
             (rnrs base)
             ((srfi srfi-1)
              #:select (every iota reduce zip)))

(define (make-bus id i)
  (list 0 id i))

(define bus-id cadr)
(define bus-arrival car)
(define bus-offset caddr)
(define (bus-arrive-at bus arrival)
  (cons arrival (cdr bus)))

(define (bus-wait bus target)
  (bus-arrive-at bus (- (bus-arrival bus) target)))

(define (closest-bus-arrival-after bus target)
  (bus-arrive-at bus (* (bus-id bus)
                        (1+ (quotient target (bus-id bus))))))

(define (parse-times times)
  (let* ((strings (string-split times #\,))
         (indexed-numbers (map make-bus (map string->number strings) (iota (length strings)))))
    (filter (lambda (e) (number? (bus-id e))) indexed-numbers)))

(define (parse-input input)
  (cons (string->number (car input))
        (parse-times (cadr input))))

(define test-data
  '("939"
    "7,13,x,x,59,x,31,19"))

(define (min-by key coll)
  (reduce (lambda (a b) (if (< (key a) (key b)) a b)) '() coll))

(define (read-data)
  (call-with-input-file "input"
    (lambda (port) (string-split (get-string-all port) #\newline))))

(define (part1 input)
  (define target (car input))
  (define bus-schedules (map (lambda (bus) (closest-bus-arrival-after bus target)) (cdr input)))
  (let ((closest-bus (min-by bus-arrival (map (lambda (bus) (bus-wait bus target)) bus-schedules))))
    (* (bus-id closest-bus) (bus-arrival closest-bus))))

(assert (= 295 (part1 (parse-input test-data))))

(define (part2 input)
  (find-aligned-schedule (cdr input)))

(define (test-part2 input)
  (part2 (parse-input (list "0" input))))

(define (find-aligned-schedule buses)
  (let loop ((bus (cadr buses))
             (rest (cddr buses))
             (step (bus-id (car buses)))
             (timestamp 0))
    (if (zero? (remainder (+ timestamp (bus-offset bus)) (bus-id bus)))
        (if (null? rest)
            timestamp
            (loop (car rest) (cdr rest) (* step (bus-id bus)) timestamp))
        (loop bus rest step (+ timestamp step)))))

(assert (= 1068781    (test-part2 (cadr test-data))))
(assert (= 3417       (test-part2 "17,x,13,19")))
(assert (= 754018     (test-part2 "67,7,59,61")))
(assert (= 779210     (test-part2 "67,x,7,59,61")))
(assert (= 1261476    (test-part2 "67,7,x,59,61")))
(assert (= 1202161486 (test-part2 "1789,37,47,1889")))

(let ((input (parse-input (read-data))))
  (format #t "Part 1: ~A~%" (part1 input))
  (format #t "Part 2: ~A~%" (part2 input)))
