(ns day01
  (:require [clojure.string :as s]
            [clojure.edn :as edn]))

(defn sum [col]
  (apply + col))

(defn parse-group [group]
  (->> group
       s/split-lines
       (map edn/read-string)
       sum))

(defn parse-input [input]
  (->> (s/split input #"\n\n")
       (map parse-group)))

(defn part1 [input]
  (apply max input))

(defn part2 [input]
  (->> input
       (sort >)
       (take 3)
       sum))

(comment
  (def input (slurp "example"))
  (def parsed-input (parse-input input))

  (= 24000 (part1 parsed-input))
  (= 45000 (part2 parsed-input))
  )

;; Invoke with clj -X day01/main
(defn main [& _args]
  (let [input (parse-input (slurp "input"))]
    (do
      (println (str "Part 1: " (part1 input)))
      (println (str "Part 2: " (part2 input))))))
