(ns day03
  (:require
   [clojure.string :as str]
   [clojure.set :as set]))

(defn split-compartments [line]
  (->> line
       (split-at (/ (count line) 2))
       (map set)))

(defn parse-input [input]
  (str/split-lines input))

(defn find-shared [sets]
  (-> (apply set/intersection sets)
      seq
      first))

(def score-table
  (zipmap (concat (range (int \a) (inc (int \z)))
                  (range (int \A) (inc (int \Z))))
          (map inc (range))))

(defn score-item [item]
  (score-table (int item)))

(defn score-rucksack [rucksack]
  (->> rucksack
       split-compartments
       find-shared
       score-item))

(defn part1 [input]
  (->> input
       (map score-rucksack)
       (apply +)))

(defn score-group [group]
  (->> group
       (map set)
       find-shared
       score-item))

(defn part2 [input]
  (->> input
       (partition 3 3)
       (map score-group)
       (apply +)))

(comment
  (def input (parse-input (slurp "example")))

  (= 157 (part1 input))

  (= 18 (part2 (take 3 input)))
  (= 52 (part2 (drop 3 input)))
  (= 70 (part2 input))
  )

(defn main [& _args]
  (let [input (parse-input (slurp "input"))]
    (do
      (println (str "Part 1: " (part1 input)))
      (println (str "Part 2: " (part2 input))))))
