(ns day04
  (:require
   [clojure.edn :as edn]
   [clojure.set :as set]
   [clojure.string :as str]))

(defn inclusive-range [from to]
  (range from (inc to)))

(defn parse-sections [line]
  (let [[from to] (map edn/read-string (str/split line #"-"))]
    (set (inclusive-range from to))))

(defn parse-line [line]
  (->> (str/split line #",")
       (map parse-sections)))

(defn parse-input [input]
  (->> input
       str/split-lines
       (map parse-line)))

(defn completely-contained? [[s1 s2]]
  (or (set/subset? s1 s2)
      (set/subset? s2 s1)))

(defn partially-overlapping? [[s1 s2]]
  (not-empty (set/intersection s1 s2)))

(defn solution [p]
  (fn [input]
    (->> input
         (filter p)
         count)))

(def part1 (solution completely-contained?))
(def part2 (solution partially-overlapping?))

(comment
  (def input (parse-input (slurp "example")))

  (= 2 (part1 input))
  (= 4 (part2 input))
  (main))

(defn main [& _args]
  (let [input (parse-input (slurp "input"))]
    (do
      (println (str "Part 1: " (part1 input)))
      (println (str "Part 2: " (part2 input))))))
