(ns day05
  (:require
   [clojure.edn :as edn]
   [clojure.string :as str]))

(defn stack-count [counters]
  (-> counters
       (str/split #" ")
       last
       edn/read-string))

(defn read-stack [stacks n]
  (->> stacks
       (map #(get %1 (inc (* 4 n))))
       (filter #(not= %1 \space))))

(defn parse-stacks [stacks]
  (let* [lines (str/split-lines stacks)
         stacks (take (dec (count lines)) lines)]
    (->> lines
         last
         stack-count
         range
         (map #(read-stack stacks %1))
         vec)))

(defn zero-index [[n from to]]
  [n (dec from) (dec to)])

(defn parse-command [command]
  (->> command
       (re-matches #"move (\d+) from (\d+) to (\d+)")
       (drop 1)
       (map edn/read-string)
       zero-index))

(defn parse-commands [commands]
  (->> commands
       str/split-lines
       (map parse-command)))

(defn parse-input [input]
  (let [[stacks commands] (str/split input #"\n\n")]
    [(parse-stacks stacks) (parse-commands commands)]))

(defn move-as-group [stacks [n from to]]
  (let [s (get stacks from)]
    (-> stacks
        (update-in [from] #(drop n %1))
        (update-in [to] #(concat (take n s) %1)))))

(defn move-by-one [stacks [n from to]]
  (loop [n n stacks stacks]
    (if (= n 0)
      stacks
      (recur (dec n) (move-as-group stacks [1 from to])))))

(defn stack-tops [stacks]
  (->> stacks
       (map first)
       (apply str)))

(defn solution [reduction]
  (fn [[stacks commands]]
    (-> (reduce reduction stacks commands)
        stack-tops)))

(def part1 (solution move-by-one))
(def part2 (solution move-as-group))

(comment
  (def input (parse-input (slurp "example")))

  (= "CMZ" (part1 input))
  (= "MCD" (part2 input))
  (main))

(defn main [& _args]
  (let [input (parse-input (slurp "input"))]
    (do
      (println (str "Part 1: " (part1 input)))
      (println (str "Part 2: " (part2 input))))))
