(ns day06
  (:require
   [clojure.string :as str]))

(defn parse-input [input]
  (first (str/split-lines input)))

(defn solution [marker-length]
  (fn [input]
    (->> input
         (partition marker-length 1)
         (map-indexed (fn [idx item] [idx (set item)]))
         (filter (fn [[_ item]] (= marker-length (count item))))
         first ;; First set of marker-length distinct letters
         first ;; The index
         (+ marker-length))))

(def part1 (solution 4))
(def part2 (solution 14))

(comment
  (= 7 (part1 "mjqjpqmgbljsphdztnvjfqwrcgsmlb"))
  (= 5 (part1 "bvwbjplbgvbhsrlpgdmjqwftvncz"))
  (= 6 (part1 "nppdvjthqldpwncqszvftbrmjlhg"))
  (= 10 (part1 "nznrnfrfntjfmvfwmzdfjlvtqnbhcprsg"))
  (= 11 (part1 "zcfzfwzzqfrljwzlrfnpqdbhtmscgvjw"))

  (= 19 (part2 "mjqjpqmgbljsphdztnvjfqwrcgsmlb"))
  (= 23 (part2 "bvwbjplbgvbhsrlpgdmjqwftvncz"))
  (= 23 (part2 "nppdvjthqldpwncqszvftbrmjlhg"))
  (= 29 (part2 "nznrnfrfntjfmvfwmzdfjlvtqnbhcprsg"))
  (= 26 (part2 "zcfzfwzzqfrljwzlrfnpqdbhtmscgvjw"))
  (main))

(defn main [& _args]
  (let [input (parse-input (slurp "input"))]
    (do
      (println (str "Part 1: " (part1 input)))
      (println (str "Part 2: " (part2 input))))))
