module Day07

let directoriesAbove parts =
    seq {
        for i in 0 .. (List.length parts) - 1 do
            yield parts.[0..i]
    }
    |> Seq.map (fun parts -> System.IO.Path.Join(Array.ofList parts))

let insertFile sizes dir size =
    let reduction sizes path =
        sizes
        |> Map.change path (fun v ->
            match v with
            | Some x -> Some(x + size)
            | None -> Some size)

    dir |> directoriesAbove |> Seq.fold reduction sizes

let (|Prefix|_|) (p: string) (s: string) =
    if s.StartsWith(p) then
        Some(s.Substring(p.Length))
    else
        None

let parseInput lines =
    let pop (cwd, sizes) =
        let count = List.length cwd
        (cwd.[0 .. count - 2], sizes)

    let cd (cwd, sizes) path = (cwd @ [ path ], sizes)

    let file (cwd, sizes) (line: string) =
        let [| size; _name |] = line.Split(" ")
        let sizes = insertFile sizes cwd (int size)
        (cwd, sizes)

    let reduction state (line: string) =
        let (cwd, sizes) = state

        match line.TrimEnd() with
        | Prefix "$ cd " path ->
            match path with
            | "/" -> ([ "/" ], sizes)
            | ".." -> pop state
            | path -> cd state path
        | Prefix "$ ls" _path -> state
        | Prefix "dir " _path -> state
        | s -> file state s

    let (_, sizes) = Seq.fold reduction ([ "/" ], Map []) lines
    sizes

let parse (input: string) =
    input.Split('\n') |> Seq.filter (fun s -> not (s = "")) |> parseInput

let part1 (input: string) =
    input |> parse |> Map.filter (fun _ v -> v < 100000) |> Map.values |> Seq.sum

let part2 (input: string) =
    let sizes = input |> parse
    let needed = 30000000 - (70000000 - sizes["/"])

    sizes
    |> Map.filter (fun _ v -> v >= needed)
    |> Map.values
    |> Seq.sort
    |> Seq.head
