let parse (lines: seq<string>) =
    let charToInt (c: char) = int c - int '0'

    let rows = Seq.length lines
    let cols = lines |> Seq.head |> Seq.length

    let arr = Array2D.zeroCreate<int> rows cols

    lines
    |> Seq.iteri (fun ri c -> c |> Seq.iteri (fun ci v -> arr.[ri, ci] <- charToInt v))

    arr

let dimensions arr =
    (Array2D.length1 arr, Array2D.length2 arr)

let pointsInDirection (x, y) (mx, my) (dx, dy) =
    (x + dx, y + dy)
    |> Seq.unfold (fun (x, y) ->
        if x < 0 || y < 0 || x >= mx || y >= my then
            None
        else
            Some((x, y), (x + dx, y + dy)))

let border arr (x, y) =
    let (mx, my) = dimensions arr
    x = 0 || y = 0 || x = mx - 1 || y = my - 1

let directions = [ (1, 0); (0, 1); (-1, 0); (0, -1) ]

let eachPoint arr =
    let (mx, my) = dimensions arr

    seq {
        for i in 0 .. mx - 1 do
            for j in 0 .. my - 1 -> (i, j)
    }

let part1 arr =
    let (mx, my) = dimensions arr

    arr
    |> eachPoint
    |> Seq.map (fun (x, y) ->
        let v = arr.[x, y]

        border arr (x, y)
        || directions
           |> Seq.exists (fun direction ->
               direction
               |> pointsInDirection (x, y) (mx, my)
               |> Seq.takeWhile (fun (ax, ay) -> arr.[ax, ay] < v)
               |> Seq.exists (border arr)))
    |> Seq.filter id
    |> Seq.length

let viewingDistance arr (x, y) =
    let (mx, my) = dimensions arr
    let v = arr.[x, y]

    directions
    |> Seq.map (fun direction ->
        let sight =
            pointsInDirection (x, y) (mx, my) direction
            |> Seq.takeWhile (fun (x, y) -> arr.[x, y] < v)

        let count = Seq.length sight
        if Seq.exists (border arr) sight then count else count + 1)

let scenicScore arr c = viewingDistance arr c |> Seq.reduce (*)

let part2 arr =
    arr |> eachPoint |> Seq.map (scenicScore arr) |> Seq.max

open System.IO

let parsed = fsi.CommandLineArgs.[1] |> File.ReadAllLines |> parse
printfn "Part 1: %A" (part1 parsed)
printfn "Part 2: %A" (part2 parsed)
