let newState length = List.init length (fun _ -> (0, 0))

let expand dir count = Seq.init count (fun _ -> dir)

let parse lines =
    lines
    |> Seq.map (fun (l: string) ->
        let [| direction; count |] = l.Split ' '

        let direction =
            match direction with
            | "R" -> (1, 0)
            | "L" -> (-1, 0)
            | "U" -> (0, 1)
            | "D" -> (0, -1)
            | _ -> failwith "unreachable"

        expand direction (int count))
    |> Seq.concat

let closeEnough (hx, hy) (tx, ty) =
    tx + 1 >= hx && tx - 1 <= hx && ty + 1 >= hy && ty - 1 <= hy

let pointDirection x1 x2 =
    match x1 with
    | _ when x1 > x2 -> 1
    | _ when x1 = x2 -> 0
    | _ when x1 < x2 -> -1
    | _ -> failwith "unreachable"

let inDirection (px, py) (dx, dy) = (px + dx, py + dy)

let rec move state direction =
    let moveClose (hx, hy) (tx, ty) =
        (pointDirection hx tx, pointDirection hy ty) |> inDirection (tx, ty)

    let rec follow state =
        match state with
        | []
        | _ :: [] -> state
        | h :: t :: _rest when closeEnough h t -> state
        | h :: t :: rest -> h :: follow ((moveClose h t) :: rest)

    let h :: rest = state
    follow (inDirection h direction :: rest)

let solution length input =
    input
    |> Seq.scan move (newState length)
    |> Seq.map List.last
    |> Seq.distinct
    |> Seq.length

let part1 = solution 2
let part2 = solution 10

open System.IO

let parsed = fsi.CommandLineArgs.[1] |> File.ReadAllLines |> parse
printfn "Part 1: %A" (part1 parsed)
printfn "Part 2: %A" (part2 parsed)
