type Instruction =
    | Noop
    | Add of int

type Computer = { cc: int; register: int }

let parse lines =
    let parseAdd (line: string) =
        let [| _; x |] = line.Split(' ')
        Add(int x)

    lines
    |> Seq.map (fun x ->
        match x with
        | "noop" -> Instruction.Noop
        | x -> parseAdd x)

let cycles =
    let expandInstruction i =
        match i with
        | Noop -> [ Noop ]
        | Add n -> [ Noop; Add n ]

    Seq.map expandInstruction >> Seq.concat

let execute computer instruction =
    let cc = computer.cc + 1

    match instruction with
    | Noop -> { computer with cc = cc }
    | Add n ->
        { cc = cc
          register = computer.register + n }

let execution = cycles >> Seq.scan execute { cc = 1; register = 1 }

let part1 instructions =
    let signalStrength computer = computer.cc * computer.register

    let poi = [ 20; 60; 100; 140; 180; 220 ]

    instructions
    |> execution
    |> Seq.where (fun c -> Seq.contains c.cc poi)
    |> Seq.take (Seq.length poi)
    |> Seq.map signalStrength
    |> Seq.sum

let part2 instructions =
    let printPixel c =
        let x = (c.cc - 1) % 40

        if x <= c.register + 1 && x >= c.register - 1 then
            printf "#"
        else
            printf "."

    let lastColumn i = i % 40 = 39

    instructions
    |> execution
    |> Seq.take (40 * 6)
    |> Seq.iteri (fun i c ->
        printPixel c

        if lastColumn i then
            printfn "")

open System.IO

let parsed = fsi.CommandLineArgs.[1] |> File.ReadAllLines |> parse
printfn "Part 1: %A" (part1 parsed)
printfn "Part 2:"
part2 parsed
