defmodule Day1 do
  def part1(lines) do
    solution(lines, &match_starting_at/2)
  end

  def part2(lines) do
    solution(lines, &match_starting_at_ext/2)
  end

  defp solution(lines, match) do
    lines
    |> Enum.map(&line_numbers(&1, match))
    |> Enum.map(&assemble_numbers/1)
    |> Enum.sum()
  end

  defp assemble_numbers([]), do: 0
  defp assemble_numbers(l), do: List.first(l) * 10 + List.last(l)

  defp line_numbers(line, match_f) do
    line
    |> String.codepoints()
    |> Enum.with_index()
    |> Enum.map(fn {_, i} -> match_f.(line, i) end)
    |> Enum.filter(& &1)
  end

  defp match_starting_at(line, i) do
    case String.slice(line, i..-1) do
      "1" <> _ -> 1
      "2" <> _ -> 2
      "3" <> _ -> 3
      "4" <> _ -> 4
      "5" <> _ -> 5
      "6" <> _ -> 6
      "7" <> _ -> 7
      "8" <> _ -> 8
      "9" <> _ -> 9
      _ -> nil
    end
  end

  defp match_starting_at_ext(line, i) do
    case String.slice(line, i..-1) do
      "one" <> _ -> 1
      "two" <> _ -> 2
      "three" <> _ -> 3
      "four" <> _ -> 4
      "five" <> _ -> 5
      "six" <> _ -> 6
      "seven" <> _ -> 7
      "eight" <> _ -> 8
      "nine" <> _ -> 9
      _ -> match_starting_at(line, i)
    end
  end
end

lines = IO.stream() |> Enum.map(& &1)

p1 = Day1.part1(lines)
p2 = Day1.part2(lines)

IO.puts("Part 1: #{p1}")
IO.puts("Part 2: #{p2}")
