{-# LANGUAGE OverloadedStrings #-}

import Data.Text qualified as T
import Data.Text.IO qualified as T.IO
import Text.Printf (printf)

type Set = (Integer, Integer, Integer)

type Game = (Integer, [Set])

emptySet = (0, 0, 0)

part1 :: [Game] -> Integer
part1 = sum . map fst . filter (all setWithinLimits . snd)

part2 :: [Game] -> Integer
part2 = sum . map (setPower . minimumSet . snd)

minimumSet :: [Set] -> Set
minimumSet ss = (maximum $ map first ss, maximum $ map second ss, maximum $ map third ss)
  where
    first (r, _, _) = r
    second (_, g, _) = g
    third (_, _, b) = b

setWithinLimits :: Set -> Bool
setWithinLimits (r, g, b) = r <= 12 && g <= 13 && b <= 14

addSets :: Set -> Set -> Set
addSets (r1, g1, b1) (r2, g2, b2) = (r1 + r2, g1 + g2, b1 + b2)

sumSets :: [Set] -> Set
sumSets = foldl addSets emptySet

setPower :: Set -> Integer
setPower (r, g, b) = r * g * b

parseInput :: T.Text -> [Game]
parseInput = map parseGame . T.lines

parseGame :: T.Text -> Game
parseGame s = (parseId $ head parts, parseSets $ last parts)
  where
    parts = T.splitOn ":" s

parseId :: T.Text -> Integer
parseId = read . T.unpack . last . T.words

parseSets :: T.Text -> [Set]
parseSets = map parseSet . T.splitOn ";"

parseSet :: T.Text -> Set
parseSet = sumSets . map parseDraw . T.splitOn ","

parseDraw :: T.Text -> Set
parseDraw s = setOf count color
  where
    setOf n "red" = (n, 0, 0)
    setOf n "green" = (0, n, 0)
    setOf n "blue" = (0, 0, n)
    parts = T.words s
    color = last parts
    count = read $ T.unpack $ head parts

main :: IO ()
main = do
  contents <- T.IO.getContents
  let parsed = parseInput contents
  putStr $ printf "Part 1: %d\n" $ part1 parsed
  putStr $ printf "Part 2: %d\n" $ part2 parsed
