{-# LANGUAGE OverloadedStrings #-}
module Main where

import qualified Data.Text as T
import qualified Data.Text.IO as TIO

import Data.Digits (digits, unDigits)
import Data.Attoparsec.Text

type Time = Int
type Distance = Int
type Input = [(Time, Distance)]

main :: IO ()
main = do
  input <- TIO.getContents >>= parseInput
  putStrLn ("Part 1: " <> show (part1 input))
  putStrLn ("Part 2: " <> show (part2 input))

parseInput :: T.Text -> IO Input
parseInput s = case parseOnly inputParser s of
  Right i -> pure i
  Left e -> error e

inputParser :: Parser Input
inputParser = do
  times <- string "Time: " *> numList
  distances <- string "Distance: " *> numList <* skipSpace <* endOfInput
  return $ zip times distances

numList :: Parser [Int]
numList = skipSpace *> sepBy1 decimal skipSpace <* endOfLine

travelledDistance :: Int -> Int -> Int
travelledDistance charge maxTime = (maxTime - charge) * charge

winningOptions :: (Time, Distance) -> Int
winningOptions (t, d) = length $ filter (\x -> travelledDistance x t > d) [1..t - 1]

part1 :: Input -> Int
part1 = product . map winningOptions

part2 :: Input -> Int
part2 i = winningOptions (joinNumbers $ map fst i, joinNumbers $ map snd i)

joinNumbers :: [Int] -> Int
joinNumbers = unDigits 10 . foldMap (digits 10)
