(ns day09
  (:require [clojure.string :as s]
            [clojure.edn :as edn]))

(defn parse-line [line]
  (->> (s/split line #" ")
       (map edn/read-string)))

(defn parse-input [input]
  (->> (s/split-lines input)
       (map parse-line)))

(defn differences [steps]
  (map - (rest steps) steps))

(defn all-zeroes? [steps]
  (every? zero? steps))

(defn expand-line [steps]
  (let [diffs (differences steps)]
    (if (all-zeroes? diffs)
      (last steps)
      (let [added (expand-line diffs)]
        (+ added (last steps))))))

(defn part1 [steps]
  (reduce + (pmap expand-line steps)))

(defn part2 [steps]
  (part1 (pmap reverse steps)))

(comment
  (= [1 2 3]
     (parse-line "1 2 3"))
  (= [[1 2 3] [4 5 6]]
     (parse-input "1 2 3\n4 5 6"))
  (differences [1 2 3])
  (= [3 3 3 3 3]
     (differences [0 3 6 9 12 15]))
  (= [0 0 0 0]
     (differences [3 3 3 3 3]))
  (= 18
     (expand-line [0 3 6 9 12 15]))
  (= 28
     (expand-line [1 3 6 10 15 21]))
  (= 68
     (expand-line [10 13 16 21 30 45]))

  (let [example (parse-input (slurp "example"))]
    (and (= 114 (part1 example))
         (= 2 (part2 example)))))

;; Invoke with clj -X day09/main
;; Or with bb --init day09.clj -e '(day09/main)'
(defn -main [& args]
  (let [input (parse-input (slurp "input"))
        [p1 p2] (map #(%1 input) [part1 part2])]
    (do
      (println (str "Part 1: " p1))
      (println (str "Part 2: " p2)))))
